# coding: utf-8
# Copyright (C) 2022, [Breezedeus](https://github.com/breezedeus).
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
# Credits: adapted from https://github.com/PaddlePaddle/PaddleOCR

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import copy

__all__ = ['build_post_process']

from .rec_postprocess import (
    CTCLabelDecode,
    AttnLabelDecode,
    SRNLabelDecode,
    DistillationCTCLabelDecode,
    TableLabelDecode,
    NRTRLabelDecode,
    SARLabelDecode,
    SEEDLabelDecode,
)


def build_post_process(config, global_config=None):
    support_dict = [
        'CTCLabelDecode',
        'AttnLabelDecode',
        'ClsPostProcess',
        'SRNLabelDecode',
        'DistillationCTCLabelDecode',
        'TableLabelDecode',
        'DistillationDBPostProcess',
        'NRTRLabelDecode',
        'SARLabelDecode',
        'SEEDLabelDecode',
    ]

    config = copy.deepcopy(config)
    module_name = config.pop('name')
    if module_name == "None":
        return
    if global_config is not None:
        config.update(global_config)
    assert module_name in support_dict, Exception(
        'post process only support {}'.format(support_dict)
    )
    module_class = eval(module_name)(**config)
    return module_class
